@@ -1,10 +1,9 @@
 package SeeAlso::Response;
 
-use JSON;
-my $json = JSON->new(autoconv=>0);
+use JSON::XS qw(encode_json);
 
 use vars qw( $VERSION );
-$VERSION = "0.40";
+$VERSION = "0.53";
 
 =head1 NAME
 
@@ -29,11 +28,10 @@ method is used.
 sub new {
     my ($class, $query, $completions, $descriptions, $urls) = @_;
 
-    $query = "" unless defined $query;
     if (UNIVERSAL::isa( $query, 'SeeAlso::Identifier' )) {
         $query = $query->normalized();
     } else {
-        $query = "$query"; # convert to string
+        $query = defined $query ? "$query" : ""; # convert to string
     }
 
     my $self = bless {
@@ -104,7 +102,9 @@ sub hasQuery {
 
 Return the response in JSON format and a non-mandatory 
 callback wrapped around. There is no test whether the 
-callback name is valid so far.
+callback name is valid so far. The encoding will not
+be changed, please only feed response objects with
+UTF-8 strings to get UTF-8 JSON with this method!
 
 =cut
 
@@ -120,7 +120,7 @@ sub toJSON {
         $self->{urls}
     ];
 
-    my $jsonstring = $json->encode( $response );
+    my $jsonstring = encode_json($response);
     return $callback ? "$callback($jsonstring);" : $jsonstring;
 }
 
